#include "ExtractorFromDeposits.h"

#include "DataFormats/Common/interface/Handle.h"
#include "FWCore/MessageLogger/interface/MessageLogger.h"
#include "DataFormats/RecoCandidate/interface/IsoDepositDirection.h"

using namespace edm;
using namespace std;
using namespace reco;
using namespace muonisolation;
using reco::isodeposit::Direction;

ExtractorFromDeposits::ExtractorFromDeposits( const ParameterSet& par, edm::ConsumesCollector && iC ) :
  theCollectionToken(iC.consumes<reco::IsoDepositMap>(par.getParameter<edm::InputTag>("IsolationCollectionTag")))
{ }

void ExtractorFromDeposits::fillVetos (const edm::Event & ev,
    const edm::EventSetup & evSetup, const reco::TrackCollection & muons)
{ }

IsoDeposit ExtractorFromDeposits::deposit(const Event & event,
    const EventSetup & eventSetup, const Track & muon) const
{
  static const std::string metname = "RecoMuon|ExtractorFromDeposits";
  Handle<reco::IsoDepositMap> depMap;
  event.getByToken(theCollectionToken, depMap);

  LogWarning(metname)<<"Call this method only if the original muon track collection is lost";

  // double vtx_z = muon.vz();
  reco::isodeposit::Direction muonDir(muon.eta(), muon.phi());

  typedef reco::IsoDepositMap::const_iterator iterator_i;
  typedef reco::IsoDepositMap::container::const_iterator iterator_ii;
  iterator_i depI = depMap->begin();
  iterator_i depIEnd = depMap->end();
  for (; depI != depIEnd; ++depI){
    iterator_ii depII = depI.begin();
    iterator_ii depIIEnd = depI.end();
    for (; depII != depIIEnd; ++depII){
      reco::isodeposit::Direction depDir(depII->direction());
      if (muonDir.deltaR(depDir) < 1.e-6) return *depII;
    }
  }

  return IsoDeposit();
}

IsoDeposit ExtractorFromDeposits::deposit(const Event & event,
    const EventSetup & eventSetup, const TrackRef & muon) const
{
  Handle<reco::IsoDepositMap> depMap;
  event.getByToken(theCollectionToken, depMap);

  return (*depMap)[muon];
}
